package PJlib

  block CapacityConverter
    extends SystemDynamics.Interfaces.GenericConverter3;
  equation
    out1 = in2/(1 - in1/in3);
  end CapacityConverter;

  block ECIRConverter
    extends SystemDynamics.Interfaces.GenericConverter4;
  equation
    out1 = in2*in3*(1-in4)/(1-in1);
  end ECIRConverter;


  model TurboMachineT 
    "machine that transfers energy between a rotor and a fluid"
    import Modelica.SIunits.*;
    parameter SpecificHeatCapacity cp = 1005.45;
    parameter Temperature T0 = 298.15;
    parameter Mass K = 0.001 "pump constant";
    parameter Real epsi = 5 "compression";
    
    Thermo.Interfaces.FluidPort_a inflow annotation(Placement(transformation(
        extent = {{-110, -10},{-90, 10}}, rotation = 0)));
    Thermo.Interfaces.FluidPort_b outflow annotation(Placement(transformation(
        extent = {{90, -10},{110, 10}}, rotation = 0)));
    Modelica.Mechanics.Rotational.Interfaces.Flange_a shaft annotation(Placement(transformation(
        extent = {{-10, 90},{10, 110}}, rotation = 0)));
    Modelica.Blocks.Interfaces.RealOutput Qdot annotation(Placement(transformation(
        extent = {{-10, -110},{10, -90}}, rotation = -90)));

    AngularVelocity omega;
  equation    
    inflow.m_flow + outflow.m_flow = 0;
    inflow.H_flow = inflow.m_flow*inflow.h;
    omega = der(shaft.phi);
    outflow.h = inflow.h;
    inflow.m_flow = K*omega;
    outflow.p = epsi*inflow.p;
    Qdot = - omega*shaft.tau;
  end TurboMachineT;
  
  model PumpT 
    extends TurboMachineT;
    annotation(
      Diagram(coordinateSystem(
              preserveAspectRatio=true,
              extent={{-100,-100},{100,100}})),
      Icon(coordinateSystem(
           preserveAspectRatio=true,
           extent={{-100,-100},{100,100}}),
           graphics={
	     Ellipse(extent={{-100,100},{100,-100}},
		     lineColor={0,0,0}),
             Line(points={{-77,64},{94,34}},
		  thickness=1,
		  color={0,0,0}),
             Line(points={{-77,-64},{94,-34}},
		  thickness=1,
		  color={0,0,0})
		     }));
  end PumpT;
  
  model TurbineT 
    extends TurboMachineT;
    annotation(
      Diagram(coordinateSystem(
              preserveAspectRatio=true,
              extent={{-100,-100},{100,100}})),
      Icon(coordinateSystem(
           preserveAspectRatio=true,
           extent={{-100,-100},{100,100}}),
           graphics={
             Line(points={{-100,40},{100,40},{60,-40},{-60,-40},{-100,40}},
		  thickness=1,
		  color={0,0,0}),
             Line(points={{0,40},{0,100}},
		  thickness=2,
		  color={0,0,0}),
             Line(points={{-100,0},{-80,0}},
		  thickness=1,
		  color={0,0,0}),
             Line(points={{80,0},{100,0}},
		  thickness=1,
		  color={0,0,0})
		     }));
  end TurbineT;

  package Hydraulik "Einfache Hydraulik-Bibliothek"
    package Interfaces
      import Modelica.SIunits.*;
      connector Port
        Pressure p;
        flow VolumeFlowRate q;
      end Port;
      connector Port_a "flow port that is used for inflow"
        extends Port;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(extent = {{-40,-40},{40,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Solid, fillColor = {0,127,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(extent = {{-100,-100},{100,100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Solid, fillColor = {0,127,255})}));
      end Port_a;
      connector Port_b "flow port that is used for outflow"
        extends Port;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(extent = {{-40,-40},{40,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Solid, fillColor = {0,63,127})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(extent = {{-100,-100},{100,100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Solid, fillColor = {0,63,127})}));
      end Port_b;
    end Interfaces;

    package Components
      import Modelica.SIunits.*;

      model FlowSource "generates a constant flow"
        parameter VolumeFlowRate Q = 0.001 "flow from A to B";
        Interfaces.Port_a inflow annotation(Placement(transformation(extent = {{-110,-10},{-90,10}}, rotation = 0)));
        Interfaces.Port_b outflow annotation(Placement(transformation(extent = {{90,-10},{110,10}}, rotation = 0)));
      equation
        inflow.q + outflow.q = 0;
        inflow.q = Q;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}})), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(extent = {{-50,-50},{50,50}}, lineColor = {0,0,0}),Polygon(points = {{12,25},{50,0},{12,-25},{12,25}}, lineColor = {0,0,0}, fillColor = {0,0,128}, fillPattern = FillPattern.Solid),Line(points = {{-100,0},{-50,0}}, color = {0,0,0}),Line(points = {{50,0},{100,0}}, color = {0,0,0})}));
      end FlowSource;

      model AmbientPressure "constant athmospheric pressure"
        parameter Pressure p0 = 100000.0 "athmospheric pressure";
        Interfaces.Port_b outflow annotation(Placement(transformation(extent = {{90,-10},{110,10}}, rotation = 0)));
      equation
        outflow.p = p0;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}})), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Line(points = {{0,0},{100,0}}),Line(points = {{20,-50},{-20,-50},{-20,50},{20,50}})}));
      end AmbientPressure;

      model LaminarFlow
        parameter Real R "flow resistance";
        Interfaces.Port_a inflow annotation(Placement(transformation(extent = {{-110,-10},{-90,10}}, rotation = 0)));
        Interfaces.Port_b outflow annotation(Placement(transformation(extent = {{90,-10},{110,10}}, rotation = 0)));
      equation
        inflow.q + outflow.q = 0;
        inflow.p - outflow.p = R * inflow.q;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}})), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Line(points = {{-100,0},{100,0}}, color = {0,0,0}),Line(points = {{-80,40},{-50,20},{-20,10},{20,10},{50,20},{80,40}}, color = {0,0,0}),Line(points = {{-80,-40},{-50,-20},{-20,-10},{20,-10},{50,-20},{80,-40}}, color = {0,0,0})}));
      end LaminarFlow;

      model HydraulicCylinder
        parameter Area A "cross section of the piston";
        parameter Pressure p0 = 100000.0 "athmospheric pressure";
        Interfaces.Port_a inflow annotation(Placement(transformation(extent = {{-60,90},{-40,110}}, rotation = 90)));
        Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90,-10},{110,10}}, rotation = 0)));
      protected
        Real v;
      equation
        flange.f = -A * (inflow.p - p0);
        v = der(flange.s);
        inflow.q = A * v;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}})), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Line(points = {{60,60},{-90,60},{-90,-60},{60,-60}}, color = {0,0,0}),Line(points = {{20,60},{20,-60}}, color = {0,0,0}),Line(points = {{-50,100},{-50,60}}, color = {0,0,0}),Line(points = {{20,0},{100,0}}, color = {0,0,0}),Rectangle(extent = {{-90,-60},{20,60}}, lineColor = {0,0,0}, fillColor = {83,233,255}, fillPattern = FillPattern.Solid)}));
      end HydraulicCylinder;

      model Pumpe
        parameter VolumeFlowRate Q = 0.001 "flow from A to B";
        parameter Real R = 1000000000.0 "flow resistance";
        PJlib.Hydraulik.Interfaces.Port_a porta1 annotation(Placement(transformation(extent = {{-110,-10},{-90,10}}, rotation = 0)));
        PJlib.Hydraulik.Interfaces.Port_b portb1 annotation(Placement(transformation(extent = {{90,-10},{110,10}}, rotation = 0)));
        PJlib.Hydraulik.Components.FlowSource flowsource1(Q = Q) annotation(Placement(visible = true, transformation(origin = {-19.943,11.6809}, extent = {{-12,-12},{12,12}}, rotation = 0)));
        PJlib.Hydraulik.Components.LaminarFlow laminarflow1(R = R) annotation(Placement(visible = true, transformation(origin = {38.1766,-19.3732}, extent = {{-12,12},{12,-12}}, rotation = -90)));
      equation
        connect(laminarflow1.outflow,flowsource1.inflow) annotation(Line(points = {{38.1766,-31.3732},{38.4615,-31.3732},{38.4615,-38.7464},{-57.5499,-38.7464},{-57.5499,11.6809},{-31.943,11.6809},{-31.943,11.6809}}));
        connect(laminarflow1.inflow,flowsource1.outflow) annotation(Line(points = {{38.1766,-7.37322},{38.1766,-7.37322},{38.1766,11.6809},{-7.94302,11.6809},{-7.94302,11.6809}}));
        connect(flowsource1.outflow,portb1) annotation(Line(points = {{-7.94302,11.6809},{98.0057,11.6809},{98.0057,9.97151},{99.1453,9.97151}}));
        connect(flowsource1.inflow,porta1) annotation(Line(points = {{-31.943,11.6809},{-97.7208,11.6809},{-97.7208,10.5413},{-99.7151,10.5413}}));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}})), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100,-100},{100,100}}), graphics = {Ellipse(rotation = 0, lineColor = {0,0,0}, fillColor = {0,0,0}, pattern = LinePattern.Solid, fillPattern = FillPattern.None, lineThickness = 0.25, extent = {{-50,-50},{50,50}}),Polygon(points = {{12,25},{50,0},{12,-25},{12,25}}, rotation = 0, lineColor = {0,0,0}, fillColor = {0,0,128}, pattern = LinePattern.Solid, fillPattern = FillPattern.Solid, lineThickness = 0.25),Rectangle(lineColor = {0,0,0}, extent = {{-100,-100},{100,100}}),Line(points = {{-100,0},{-50,0}}, rotation = 0, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25),Line(points = {{50,0},{100,0}}, rotation = 0, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25)}));
      end Pumpe;

    end Components;
  end Hydraulik;

end PJlib;

